#ifndef irdeto2_xx
#define irdeto2_xx

//#include "irdeto.h"
#include <stdlib.h>
#include <string.h>
#include "helper.h"
#include "emu.h"
#include "../common.h"
#include <openssl/des.h>

class cIrdeto2 {
private:
  DES_key_schedule ks1, ks2;
  void ScheduleKey(const unsigned char *key);
  void DES3(unsigned char *data, int mode);

  void PrepareSeed(unsigned char *seed, const unsigned char *key);
  void NanoDecrypt(unsigned char *data, int i, int len, const unsigned char *key, const unsigned char *iv);
protected:
  void Encrypt(unsigned char *data, const unsigned char *seed, const unsigned char *key, int len);
  void Decrypt(unsigned char *data, const unsigned char *seed, const unsigned char *key, int len);
  bool CalculateHash(const unsigned char *key, const unsigned char *iv, const unsigned char *data, int len);
public:
  void HEXDUMP(char *line,unsigned char *buffer, int length);
  bool ProcessECM(cEcmInfo *ecm, unsigned char *data);
  unsigned char cw[16];
  };

void cIrdeto2::HEXDUMP(char *line,unsigned char *buffer, int length)
{
	int i;
        printf("%s",line);
	for (i = 0;i < length;i++)
	{
		if (i % 16 == 0) printf("\n%04X=", i);
		printf("%02X", buffer[i]);
	}
	printf("\n");
}

void cIrdeto2::ScheduleKey(const unsigned char *key)
{
  DES_key_sched((DES_cblock *)key,&ks1);
  DES_key_sched((DES_cblock *)(key+8),&ks2);
}

void cIrdeto2::DES3(unsigned char *data, int mode)
{
  int m1, m2;
  if(mode) { m1=DES_DECRYPT; m2=DES_ENCRYPT; }
  else     { m1=DES_ENCRYPT; m2=DES_DECRYPT; }
  DES_ecb_encrypt((DES_cblock *)data,(DES_cblock *)data,&ks1,m1);
  DES_ecb_encrypt((DES_cblock *)data,(DES_cblock *)data,&ks2,m2);
  DES_ecb_encrypt((DES_cblock *)data,(DES_cblock *)data,&ks1,m1);
}

void cIrdeto2::Encrypt(unsigned char *data, const unsigned char *seed, const unsigned char *key, int len)
{
  ScheduleKey(key);
  len&=~7;
  const unsigned char *tmp=seed;
  for(int i=0; i<len; i+=8) {
    xxor(&data[i],8,&data[i],tmp); tmp=&data[i];
    DES3(&data[i],0);
    }
}

void cIrdeto2::Decrypt(unsigned char *data, const unsigned char *seed, const unsigned char *key, int len)
{
  ScheduleKey(key);
  len&=~7;
  unsigned char buf[2][8];
  int n=0;
  memcpy(buf[n],seed,8);
  for(int i=0; i<len; i+=8,data+=8,n^=1) {
    memcpy(buf[1-n],data,8);
    DES3(data,1);
    xxor(data,8,data,buf[n]);
    }
}

bool cIrdeto2::CalculateHash(const unsigned char *key, const unsigned char *iv, const unsigned char *data, int len)
{
  ScheduleKey(key);
  unsigned char cbuff[8];
  memset(cbuff,0,sizeof(cbuff));
  len-=8;
  for(int y=0; y<len; y+=8) {
    if(y<len-8) {
      xxor(cbuff,8,cbuff,&data[y]);
      //LDUMP(L_SYS_VERBOSE,cbuff,8,"3DES XOR in:");
      }
    else {
      int l=len-y;
      xxor(cbuff,l,cbuff,&data[y]);
      xxor(cbuff+l,8-l,cbuff+l,iv+8);
      //LDUMP(L_SYS_VERBOSE,cbuff,8,"3DES XOR(%d) in:",8-l);
      }
    DES3(cbuff,0);
    //LDUMP(L_SYS_VERBOSE,cbuff,8,"3DES out:");
    }
  //LDUMP(L_SYS_VERBOSE,cbuff,8,"CryptBuffer:");
  //LDUMP(L_SYS_VERBOSE,&data[len],8,"MACBuffer:");
  return memcmp(cbuff,&data[len],8)==0;
}

// -- cSystemIrd2 --------------------------------------------------------------

#define NANOLEN(_a) ((_a) ? ((_a)&0x3F)+2 : 1)


void cIrdeto2::PrepareSeed(unsigned char *seed, const unsigned char *key)
{
  unsigned char blank[16];
  memset(blank,0,16);
  Encrypt(seed,blank,key,16);
}

void cIrdeto2::NanoDecrypt(unsigned char *data, int i, int len, const unsigned char *key, const unsigned char *iv)
{
  while(i<len) {
    int l=NANOLEN(data[i+1]);
    //printf("NANODEC -> data(i) %02x  len:%d\n",data[i],l);
    switch(data[i]) {
      case 0x10:
      case 0x50: if(l==0x13 && i<=len-l) Decrypt(&data[i+3],iv,key,16); break;
      case 0x78: if(l==0x14 && i<=len-l) Decrypt(&data[i+4],iv,key,16); break;
      }
    i+=l;
    }
}

bool cIrdeto2::ProcessECM(cEcmInfo *ecm, unsigned char *data)
{
// M1=ecm_seed   M2=ecm_IV 
  int len=data[11];
  if( !((len==0x28) ||(len==0x30)) || SCT_LEN(data)<len+12) {
    //if(doLog) PRINTF(L_SYS_ECM,"bad ECM length");
	//printf("bad ECM length\n");
    return false;
    }

//I 060300 M2 87A3DBFC22B199E762C649E08F2A5E65 ; Al Jazeera Sport ricky 2013-01-06 13:40:59 -----IV
//I 060300 M1 903C3379A8698CE1D829CE82010C2F31 ; Al Jazeera Sport coscos1222 2012-12-04 21:41:11-----seed
//I 060300 06 7B4412B80604D053DE5A4A057E161A5B ; Al Jazeera Sport tonytr 2013-01-13 18:14:35 
  unsigned char ECM_IV[16];
  unsigned char ECM_Seed[16];
  unsigned char key[16];
  char hcaid[8]; 	sprintf(hcaid,"%04X%02X",ecm->caId,data[8]);
  char hkeynr[8];	sprintf(hkeynr,"%02X",data[9]);	
  
  FindKey("I",hcaid,"M1",ecm);		// Seed
  keystr_to_key(ecm->indata,ECM_Seed,ECM_Seed+8);

  FindKey("I",hcaid,"M2",ecm);		// IV 
  keystr_to_key(ecm->indata,ECM_IV,ECM_IV+8);

  FindKey("I",hcaid,hkeynr,ecm);	//op key
  keystr_to_key(ecm->indata,key,key+8);

  PrepareSeed(ECM_Seed,key);

  data+=12; 
  Decrypt(data,ECM_IV,ECM_Seed,len);
//HEXDUMP("CRYPTED ECM :",data,len);
  int i=(data[0]&7)+1;
  NanoDecrypt(data,i,len-8,key,ECM_IV);
  if (CalculateHash(ECM_Seed,ECM_IV,data-6,len+6)) {
    //HEXDUMP("Irdeto2 RAWECM",data-6,len+6);
    while(i<len-8) {
      int l=NANOLEN(data[i+1]);
      switch(data[i]) {
        case 0x78:
          memcpy(cw,&data[i+4],16);
          //ks.OK(pk);
          return true;
        }
      i+=l;
      }
    }
  else {
	//printf("hash failed\n");
    }
  return false;
}

#endif
